/*****************************************************************************
* wanpipe_lip_api.h 
* 		
*		WANPIPE(tm) Sangoma Protocol API
*
* Authors: 	David Rokhvarg <davidr@sangoma.com>
*
* Copyright:	(c) 2003-2009 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*
* November 09, 2009	David Rokhvarg	<davidr@sangoma.com>	Initial Version
*
*****************************************************************************/

#ifndef __WANPIPE_LIP_API_H_
#define __WANPIPE_LIP_API_H_ 


#include "wanpipe_includes.h"
#include "wanpipe_defines.h"
#include "wanpipe.h"
#include "wanpipe_api_iface.h"
#include "wanpipe_cdev_iface.h"


#ifdef WAN_KERNEL

#define WP_LIP_API_MAX_LEN 	4096+sizeof(wp_api_hdr_t)
#define WP_LIP_API_EVENT_MAX_LEN sizeof(wanpipe_lip_api_dev_t)+sizeof(wp_api_hdr_t)


typedef struct wanpipe_lip_api_dev {

	u32		init;
	void 	*netdev;/* pointer to netdev_t. the netdev this api dev belongs to. */
	char 	name[WAN_IFNAME_SZ];
	wan_spinlock_t lock;

	u32		used, open_cnt;
	int32_t	lip_api_span;
	int32_t	lip_api_chan;
	u8		state;

	int (*write_frame)(void *netdev, void *skb);
	int (*pipemon)(void *netdev, void *udata);

	wanpipe_api_dev_cfg_t cfg;

	wan_skb_queue_t 	wp_rx_list;

	u32		busy;
	u32 	critical;
	
	void 		*cdev;
	uint8_t		operation_mode;/* WP_TDM_OPMODE */
	uint8_t		api_mode;
	int			mtu_mru;

}wanpipe_lip_api_dev_t;


static __inline int __wp_lip_api_get_open_cnt( wanpipe_lip_api_dev_t *lip_api)
{
	return lip_api->open_cnt;
}
 

static __inline int wp_lip_api_get_open_cnt( wanpipe_lip_api_dev_t *lip_api)
{
	wan_smp_flag_t flags;
	int cnt;
	wan_spin_lock(&lip_api->lock,&flags);
	cnt=__wp_lip_api_get_open_cnt(lip_api);
	wan_spin_unlock(&lip_api->lock,&flags);
	return cnt;
}

static __inline int wp_lip_api_inc_open_cnt( wanpipe_lip_api_dev_t *lip_api)
{
	wan_smp_flag_t flags;
	int cnt;
	wan_spin_lock(&lip_api->lock,&flags);
	lip_api->open_cnt++;
	cnt = lip_api->cfg.open_cnt = lip_api->open_cnt;
	wan_spin_unlock(&lip_api->lock,&flags);
	return cnt;
}

static __inline int wp_lip_api_dec_open_cnt( wanpipe_lip_api_dev_t *lip_api)
{
	wan_smp_flag_t flags;
	int cnt;
	wan_spin_lock(&lip_api->lock,&flags);
	lip_api->open_cnt--;
	cnt = lip_api->cfg.open_cnt = lip_api->open_cnt;
	wan_spin_unlock(&lip_api->lock,&flags);
	return cnt;
}


static __inline int is_lip_api_stopped(wanpipe_lip_api_dev_t *lip_api)
{

	return wan_test_bit(0,&lip_api->busy);
}

static __inline void wp_lip_api_stop(wanpipe_lip_api_dev_t *lip_api)
{
	wan_set_bit(0,&lip_api->busy);
}

static __inline void wp_lip_api_start(wanpipe_lip_api_dev_t *lip_api)
{

	wan_clear_bit(0,&lip_api->busy);
}

static __inline int wp_lip_api_check_mtu(void* pcard, unsigned long timeslot_map, int chunksz, int *mtu)
{
	sdla_t	*card = (sdla_t*)pcard;
	int	x, num_of_channels = 0, max_channels;

	max_channels = GET_TE_CHANNEL_RANGE(&card->fe);
	for (x = 0; x < max_channels; x++) {
		if (wan_test_bit(x,&timeslot_map)){
			num_of_channels++;
		}
	}
	*mtu = chunksz * num_of_channels;
	return 0;
}

static inline u8 wp_lip_api_get_span(sdla_t *card)
{
	return card->tdmv_conf.span_no;
}

static inline int wp_lip_api_alloc_q(wanpipe_lip_api_dev_t *lip_api, wan_skb_queue_t *queue, int max_data_size, int max_q_len)
{
	netskb_t *skb;
	int i;

	if (!max_data_size || max_data_size > WP_LIP_API_MAX_LEN) {
		return -1;
	}	

	for (i=0; i < max_q_len; i++) {
		skb = wan_skb_kalloc(max_data_size);
		if (!skb) {
			if (WAN_NET_RATELIMIT()) {
				DEBUG_EVENT("%s: Error: Failed to Allocate Memory in %s():line:%d\n",
					lip_api->name, __FUNCTION__,__LINE__);
			}
			return -1;
		}
		wan_skb_init(skb,sizeof(wp_api_hdr_t));
		wan_skb_trim(skb,0);
		
		wan_skb_queue_tail(queue,skb);
	}

	return 0;
}


extern int wp_lip_api_create(wanpipe_lip_api_dev_t *lip_api_dev);
extern int wp_lip_api_delete(wanpipe_lip_api_dev_t *lip_api_dev);
extern int wp_lip_api_rx (wanpipe_lip_api_dev_t *lip_api_dev, netskb_t *skb);
extern int wp_lip_api_tx_kick(wanpipe_lip_api_dev_t *lip_api_dev);

#endif/* WAN_KERNEL */
#endif/* __WANPIPE_TDM_API_H_ */
